import sys
import unittest
import json
import os
import pandas as pd
from pandas.util.testing import assert_frame_equal

from findhelp.finder import go_search
from setup_files import setup_tests
from tear_down import tear_down_tests


def join_key(dict):
    full_key = ""
    for key, value in dict.items():
        if value != None:
            full_key += value
    return full_key


def main(out=sys.stderr, verbosity=2):
    loader = unittest.TestLoader()

    suite = loader.loadTestsFromModule(sys.modules[__name__])
    unittest.TextTestRunner(out, verbosity=verbosity).run(suite)


class BasicTests(unittest.TestCase):

    HEADER_RESULT = ["fullpath",
                     "type",
                     "folder_parent",
                     "folder_name",
                     "file_name",
                     "ext"]

    # sys.stdout = sys.__stdout__
    # print("Captured", capturedOutput.getvalue())
    @classmethod
    def setUpClass(cls):
        setup_tests()

    @classmethod
    def tearDownClass(cls):
        if os.path.exists("_test_result.json"):
            os.remove("_test_result.json")

        if os.path.exists("_test_result.csv"):
            os.remove("_test_result.csv")

        if os.path.exists("_test_result.txt"):
            os.remove("_test_result.txt")
        tear_down_tests()

    def test_data_frame(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "df"
        }

        base_result = pd.DataFrame(
            [
                ['./__search_tests_dummy_folder__',
                    'folder', '.', 'one', None, None],
                ['./__search_tests_dummy_folder__/test/dir1', 'file',
                    './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
                ['./__search_tests_dummy_folder__/test/dir2', 'file',
                    './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
                ['./__search_tests_dummy_folder__/test1', 'file',
                    './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
                ['./__search_tests_dummy_folder__/test2/test01', 'file',
                    './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
            ],
            columns=self.HEADER_RESULT
        )
        result = go_search(base_dict)

        result = result.sort_values(
            by=result.columns.tolist()).reset_index(drop=True)
        base_result = base_result.sort_values(
            by=base_result.columns.tolist()).reset_index(drop=True)
        assert_frame_equal(result, base_result)

    def test_basic_json(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "json",
            "outputfilename": "_test_result"
        }
        go_search(base_dict)
        base_result = [
            {
                "fullpath": "./__search_tests_dummy_folder__",
                "type": "folder",
                "folder_parent": ".",
                "folder_name": "one",
                "file_name": None,
                "ext": None
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test/dir1",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test",
                "folder_name": "dir1",
                "file_name": "one.txt",
                "ext": ".txt"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test/dir2",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test",
                "folder_name": "dir2",
                "file_name": "one.md",
                "ext": ".md"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test1",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__",
                "folder_name": "test1",
                "file_name": "one.csv",
                "ext": ".csv"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test2/test01",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test2",
                "folder_name": "test01",
                "file_name": "one.md",
                "ext": ".md"
            }
        ]

        with open("_test_result.json") as f:
            result = json.load(f)

        self.assertListEqual(sorted(result["results"], key=lambda x: join_key(
            x)), sorted(base_result, key=lambda x: join_key(x)))

    def test_basic_txt(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "txt",
            "outputfilename": "_test_result"
        }

        base_result = pd.DataFrame(
            [
                ['./__search_tests_dummy_folder__',
                    'folder', '.', 'one', None, None],
                ['./__search_tests_dummy_folder__/test/dir1', 'file',
                    './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
                ['./__search_tests_dummy_folder__/test/dir2', 'file',
                    './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
                ['./__search_tests_dummy_folder__/test1', 'file',
                    './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
                ['./__search_tests_dummy_folder__/test2/test01', 'file',
                    './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
            ],
            columns=self.HEADER_RESULT
        )
        go_search(base_dict)
        result = pd.read_csv("_test_result.txt", sep="\t")

        result = result.sort_values(
            by=result.columns.tolist()).reset_index(drop=True)
        base_result = base_result.sort_values(
            by=base_result.columns.tolist()).reset_index(drop=True)
        assert_frame_equal(result, base_result)

    def test_basic_csv(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "csv",
            "outputfilename": "_test_result"
        }

        base_result = pd.DataFrame(
            [
                ['./__search_tests_dummy_folder__',
                    'folder', '.', 'one', None, None],
                ['./__search_tests_dummy_folder__/test/dir1', 'file',
                    './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
                ['./__search_tests_dummy_folder__/test/dir2', 'file',
                    './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
                ['./__search_tests_dummy_folder__/test1', 'file',
                    './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
                ['./__search_tests_dummy_folder__/test2/test01', 'file',
                    './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
            ],
            columns=self.HEADER_RESULT
        )
        go_search(base_dict)
        result = pd.read_csv("_test_result.csv", sep="\t")

        result = result.sort_values(
            by=result.columns.tolist()).reset_index(drop=True)
        base_result = base_result.sort_values(
            by=base_result.columns.tolist()).reset_index(drop=True)
        assert_frame_equal(result, base_result)

    def test_basic_obj(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "obj_list"
        }
        base_result = [
            {
                "fullpath": "./__search_tests_dummy_folder__",
                "type": "folder",
                "folder_parent": ".",
                "folder_name": "one",
                "file_name": None,
                "ext": None
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test/dir1",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test",
                "folder_name": "dir1",
                "file_name": "one.txt",
                "ext": ".txt"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test/dir2",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test",
                "folder_name": "dir2",
                "file_name": "one.md",
                "ext": ".md"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test1",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__",
                "folder_name": "test1",
                "file_name": "one.csv",
                "ext": ".csv"
            },
            {
                "fullpath": "./__search_tests_dummy_folder__/test2/test01",
                "type": "file",
                "folder_parent": "./__search_tests_dummy_folder__/test2",
                "folder_name": "test01",
                "file_name": "one.md",
                "ext": ".md"
            }
        ]

        result = go_search(base_dict)
        self.assertListEqual(sorted(result, key=lambda x: join_key(
            x)), sorted(base_result, key=lambda x: join_key(x)))


if __name__ == "__main__":
    with open('test_output_results.txt', 'w') as f:
        main(f)



import sys
from io import StringIO
import unittest

from findhelp.finder import go_search
from setup_files import setup_tests
from tear_down import tear_down_tests


def main(out=sys.stderr, verbosity=2):
    loader = unittest.TestLoader()

    suite = loader.loadTestsFromModule(sys.modules[__name__])
    unittest.TextTestRunner(out, verbosity=verbosity).run(suite)


class ConsoleTests(unittest.TestCase):
    capturedOutput = StringIO()
    sys.stdout = capturedOutput

    # sys.stdout = sys.__stdout__
    # print("Captured", capturedOutput.getvalue())
    @classmethod
    def setUpClass(cls):
        setup_tests()

    @classmethod
    def tearDownClass(cls):
        tear_down_tests()

    def test_search_one(self):
        base_dict = {
            "stringsearch": "one",
        }
        result = go_search(base_dict)
        self.assertEqual(result, 5)

    def test_search_nothing(self):

        base_dict = {
            "stringsearch": "nothing_to_match",
        }
        result = go_search(base_dict)
        self.assertEqual(result, 0)


if __name__ == "__main__":
    with open('test_console_results.txt', 'w') as f:
        main(f)



import sys
import unittest

from findhelp.finder import go_search
import findhelp.finder as fhp

from setup_files import setup_tests
from tear_down import tear_down_tests


def main(out=sys.stderr, verbosity=2):
    loader = unittest.TestLoader()

    suite = loader.loadTestsFromModule(sys.modules[__name__])
    unittest.TextTestRunner(out, verbosity=verbosity).run(suite)


class BasicTests(unittest.TestCase):

    HEADER_RESULT = ["fullpath",
                     "type",
                     "folder_parent",
                     "folder_name",
                     "file_name",
                     "ext"]

    # sys.stdout = sys.__stdout__
    # print("Captured", capturedOutput.getvalue())
    @classmethod
    def setUpClass(cls):
        setup_tests()

    @classmethod
    def tearDownClass(cls):
        pass
        # reset to default
        fhp.set_ignore_folders()
        fhp.set_ignore_extensions()
        tear_down_tests()

    def test_set_ignore_folders(self):
        fhp.set_ignore_folders(["node_modules"])
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "list"
        }
        result = go_search(base_dict)
        fhp.set_ignore_folders()
        base_result = [['./__search_tests_dummy_folder__', 'folder', '.', 'one', None, None],
                       ['./__search_tests_dummy_folder__/test/dir1', 'file',
                           './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
                       ['./__search_tests_dummy_folder__/test/dir2', 'file',
                           './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
                       ['./__search_tests_dummy_folder__/test1', 'file',
                        './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
                       ['./__search_tests_dummy_folder__/test2/site-packages', 'file',
                        './__search_tests_dummy_folder__/test2', 'site-packages', 'one', ''],
                       ['./__search_tests_dummy_folder__/test2/site-packages', 'file',
                        './__search_tests_dummy_folder__/test2', 'site-packages', 'one.json', '.json'],
                       ['./__search_tests_dummy_folder__/test2/site-packages/site_inside1', 'file',
                        './__search_tests_dummy_folder__/test2/site-packages', 'site_inside1', 'sp_one', ''],
                       ['./__search_tests_dummy_folder__/test2/site-packages/site_inside1', 'file',
                        './__search_tests_dummy_folder__/test2/site-packages', 'site_inside1', 'sp_one.json', '.json'],
                       ['./__search_tests_dummy_folder__/test2/test01', 'file',
                        './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
                       ]

        self.assertListEqual(sorted(base_result), sorted(result))

    def test_reset_ignore_folders(self):
        fhp.set_ignore_folders(["site-packages"])
        fhp.set_ignore_folders()
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "list"
        }
        result = go_search(base_dict)

        base_result = [
            ['./__search_tests_dummy_folder__', 'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]

        self.assertListEqual(sorted(base_result), sorted(result))

    def test_set_ignore_extensions(self):
        fhp.set_ignore_extensions([".md", ".txt"])
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "list"
        }
        result = go_search(base_dict)
        fhp.set_ignore_extensions()

        base_result = [
            ['./__search_tests_dummy_folder__', 'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
        ]

        self.assertListEqual(sorted(base_result), sorted(result))

    def test_reset_ignore_extensions(self):
        fhp.set_ignore_extensions(["md", "txt"])
        fhp.set_ignore_extensions()
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "list"
        }
        result = go_search(base_dict)

        base_result = [
            ['./__search_tests_dummy_folder__', 'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]

        self.assertListEqual(sorted(base_result), sorted(result))


if __name__ == "__main__":
    with open('test_config_results.txt', 'w') as f:
        main(f)



import re
from typing import Union
from unidecode import unidecode

from findhelp.custom_exceptions import NotEqualLengthError


def sel_arg(arg_dict: dict, arg_search: str):
    """Selects a single element from a dictionary. Returns None if key doesn't exist

    Args:
        arg_dict (dict): Dictionary where key will be searched
        arg_search (str): Key to search for

    Returns:
        object: Element stored in arg_dict[arg_search]. None if key doesn't exist
    """
    if arg_search in arg_dict.keys():
        return arg_dict[arg_search]
    return None


def search_in_list(
        base_list: list, value: Union[str, re.Pattern], ignore_case: bool = False, ignore_accents: bool = True):
    """Search for matches in a list

    Selects the elements within a list that match with a string / regular expression pattern

    Args:
        base_list (list): List in which value will be searched
        value (Union[str, re.Pattern]): String / re.Pattern to search for
        ignore_case (bool, optional): Indicates if upper / lower case is ignored.
            Just for searching purposes, result will consider the actual characters of each coincidence. Defaults to False.
        ignore_accents (bool, optional): Indicates if accent characters must be turned into non-accented ones. 
            Just for searching purposes, result will consider the actual characters of each coincidence. Defaults to True.

    Returns:
        list[str]: List with all coincidences found, respecting ortography (e.g. accents, upper/lowercase)
    """

    if ignore_accents:
        base_new = [unidecode(
            str(element)) for element in base_list]
    else:
        base_new = base_list[:]

    # indexes
    if not ignore_case:
        if isinstance(value, re.Pattern):
            indexes = [index for index, item in enumerate(
                base_new) if value.search(str(item))]
        else:
            indexes = [index for index, item in enumerate(
                base_new) if value in str(item)]

    else:
        if isinstance(value, re.Pattern):
            indexes = [index for index, item in enumerate(
                base_new) if value.search(str(item).lower())]
        else:
            indexes = [index for index, item in enumerate(
                base_new) if value.lower() in str(item).lower()]

    return [element for index, element in enumerate(base_list) if index in indexes]


def forcedir_sep(path: str, dir_sep: str) -> str:
    """Forces separators to be a specific one

    Args:
        path (str): string path in which separators will be replaced
        dir_sep (str): separator

    Returns:
        str: path string with a single unified separator
    """
    return path.replace("\\\\", dir_sep).replace("\\", dir_sep).replace("/", dir_sep)


def join_result(output_type: str, defaultdir_sep: str, fullpath: str, type: str,
                folder_parent: str, folder_name: str, file_name: str = None,
                ext: str = None):
    """Joins a single found coincidence into a list / dict

    Args:
        output_type (str): Indicates if join must be made using 'list' or 'dict'
        defaultdir_sep (str): Separator to force all paths to have the same one
        fullpath (str): Folder in which the actual coincidence was found. fullpath = folder_parent + folder_name
        type (str): Indicates if the coincidence is a 'folder' or a 'file'
        folder_parent (str): Path to 'fullpath' parent. For filtering purposes.
        folder_name (str): Name of the folder in which the coincidence was fount. For filtering purposes.
        file_name (str, optional): Filename of the coincidence, if applicable (if type != 'folder'). Defaults to None.
        ext (str, optional): Filename extension of the coincidence, if applicable (if type != 'folder'). Defaults to None.

    Returns:
        list | dict: An list / dictionary with each of the elements, ready to join with other found coincidences
    """
    fullpath = forcedir_sep(fullpath, defaultdir_sep)
    folder_parent = forcedir_sep(folder_parent, defaultdir_sep)

    if output_type == "dict":
        return {
            "fullpath": fullpath,
            "type": type,
            "folder_parent": folder_parent,
            "folder_name": folder_name,
            "file_name": file_name,
            "ext": ext
        }
    elif output_type == "list":
        return [
            fullpath,
            type,
            folder_parent,
            folder_name,
            file_name,
            ext
        ]
    return None


def parse_line(line: list, max_len: list) -> str:
    """Builds a string from a list of strings, justifying each element using spaces

    Args:
        line (list): List of strings to justify
        max_len (list): List of lengths for each string

    Raises:
        NotEqualLengthError: Raised if len(line) and len(max_len) are different

    Returns:
        str: string resulting from justifying each of the strings within 'line',
            and putting all of them together
    """
    str_result = ""
    if len(line) != len(max_len):
        raise NotEqualLengthError(message="Lists lengths are not equal")

    for index in range(len(line)):
        str_result += str(line[index]).ljust(max_len[index])

    return str_result


def find_len(string) -> int:
    """Gets the length of a string. 0 if string is None

    Args:
        string (str): String to calculate len

    Returns:
        int: Len of the string
    """
    if string:
        string = str(string)
    else:
        string = ""

    return len(string)


def force_filename(filename: str) -> str:
    return re.sub('[^\w_.)( -]', '', filename)


def default_ignore_folders():
    """Gets default ignore folders

    Returns:
        list: Default list of folders to ignore in future searchs.
    """
    return [
        "site-packages",
        ".git",
        "$RECYCLE.BIN",
        "node_modules",
        "__pycache__",
        ".vscode",
    ]



from typing import Union
import logging
import datetime
import os
import csv
import json
import time
import yaml
from unidecode import unidecode
import re
import pandas as pd
import findhelp.custom_utils as custom_utils

from findhelp.custom_exceptions import NotValidDirectoryError
from findhelp.custom_exceptions import NotValidArgumentError
from findhelp.custom_exceptions import MissingFilename


main_logger = logging.getLogger(__name__)
main_logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(message)s")
now = datetime.datetime.now()
date_today = now.strftime("%Y%m%d")
date_now = now.strftime("%Y%m%d_%H_%M_%S")

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)

main_logger.addHandler(stream_handler)


# For printing
HEADER_RESULT = ["fullpath",
                 "type",
                 "folder_parent",
                 "folder_name",
                 "file_name",
                 "ext"]

CONFIG_PATH = os.path.join(os.path.dirname(
    os.path.realpath(__file__)), "config.yaml")


def __search_elements(
        search_type: str, string_search: Union[str, re.Pattern] = "", path: str = ".", dir_sep: str = "/",
        ignore_case: bool = False, ignore_accents: bool = False, ignore_folders: list = None, results: list = None,
        ext_exclude: list = None, ext_include: list = None, output: str = "console"):
    """Recursively searches for coincidences of files / folders in the specified 'path'. Called from '__search'

    Args:
        search_type (str): Indicates what to search for {files, folders, both}
        string_search (Union[str, re.Pattern], optional): The string to look for
        path (str, optional): Root path to look for. Defaults to ".".
        dir_sep (str, optional): Just for results purposes: which separator use for paths. Defaults to "/".
        ignore_case (bool, optional): Indicates whether or not searching is case-sensitive. Defaults to False.
        ignore_accents (bool, optional): Ignore accents. Defaults to False.
        ignore_folders (list, optional): Folders to ignore while searching. Defaults to None.
        results (list, optional): List to append the found results. Defaults to None.
        ext_exclude (list, optional): List of extensions to exclude. Defaults to None.
        ext_include (list, optional): List of extensions to search.
            If different than None, searching ignores 'ext_exclude' and considers only 'ext_include'. 
            Defaults to None.
        output (str): {console,txt,csv,json,df,obj_list,list}
                Indicates if results appears on console or are written to a file.
                Defaults to "console".

    Returns:
        _type_: _description_
    """
    # it seems double, but otherway
    if results == None:
        results = []

    if output in ["console", "list"]:
        output_type = "list"
    else:
        output_type = "dict"

    if os.path.split(path)[1] not in ignore_folders:
        for dirpath, dir_names, file_names in os.walk(path):
            dir_names[:] = [d for d in dir_names if d not in ignore_folders]

            if search_type in ("folders", "both") and not ext_include:
                # folders
                dir_founds = custom_utils.search_in_list(
                    dir_names, string_search, ignore_case, ignore_accents)

                filtered_dir_founds = [
                    found for found in dir_founds if found.lower() not in ignore_folders]

                for found in filtered_dir_founds:
                    results.append(custom_utils.join_result(output_type, dir_sep,
                                                            dirpath, "folder", os.path.dirname(dirpath), found))

            # files
            if search_type in ("files", "both"):
                founds = custom_utils.search_in_list(
                    file_names, string_search, ignore_case, ignore_accents)

                if ext_include:
                    filtered_founds = [found for found in founds if os.path.splitext(
                        found)[1].lower() in ext_include]
                elif ext_exclude:
                    filtered_founds = [found for found in founds if os.path.splitext(
                        found)[1].lower() not in ext_exclude]
                else:
                    filtered_founds = founds

                for found in filtered_founds:
                    tempext = os.path.splitext(found)[1]
                    results.append(custom_utils.join_result(output_type, dir_sep,
                                                            dirpath, "file", os.path.dirname(dirpath), os.path.basename(dirpath), found, tempext))

    return results


def __search(
        path: str, string_search: str, ignore_case: bool, ignore_accents: bool, reg_exp: bool,
        ext: Union[str, list], all: bool, only_files: bool, only_dirs: bool, dir_sep: str,
        output: str):
    """Called from 'go_search'. Defines what extensions / folders ignore and / or consider.
        and calls __search_elements to get a list of coincidences

    Args:
        path (str): Root path to look for
        string_search (str): The string to look for
        ignore_case (bool): Indicates whether or not searching is case-sensitive
        ignore_accents (bool): Ignore accents
        reg_exp (bool): Searches by regular expresion
        ext (Union[str, list]): List of extensions to limit the search
        all (bool): Searches on all folders, ignores 'ignore_folders' switch on 'config.yaml'
        only_files (bool): Searches only for files (not directories)
        only_dirs (bool): Searches only for directories (not files)
        dir_sep (str): Just for results purposes: which separator use for paths
        output (str): {console,txt,csv,json,df,obj_list,list}
                Indicates if results appears on console or are written to a file        

    Raises:
        NotADirectoryError: Raised if path is not a directory
        NotValidDirectoryError: Raised if path to directory doesn't really exist

    Returns:
        list[str]: List of strings with found coincidences
    """
    if not os.path.isdir(path):
        raise NotADirectoryError
    elif not os.path.exists(path):
        raise NotValidDirectoryError(
            message="Specified directory is not valid")

    string_search = unidecode(
        string_search) if ignore_accents else string_search

    string_search = re.compile(
        f"{string_search}") if reg_exp else string_search

    _config = __config()

    ignore_folders = custom_utils.sel_arg(_config, "ignore_folders")
    if ignore_folders == None or all:
        ignore_folders = []

    ext_exclude = custom_utils.sel_arg(_config, "ignore_extensions")

    if ext_exclude != None:
        if isinstance(ext_exclude, list):
            if len(ext_exclude) == 0:
                ext_exclude = None
            else:
                ext_exclude = [f".{str(ext)}".replace("..", ".").lower()
                               for ext in ext_exclude]
    if ext:
        ext_include = [f".{ext.lower()}".replace("..", ".") for ext in ext]
    else:
        ext_include = None

    results = []

    if only_dirs:
        results += __search_elements("folders", string_search, path,
                                     dir_sep, ignore_case, ignore_accents, ignore_folders, output=output)
        return results

    if only_files:
        results += __search_elements("files", string_search, path, dir_sep, ignore_case, ignore_accents, ignore_folders,
                                     ext_exclude=ext_exclude, ext_include=ext_include, output=output)
        return results

    results += __search_elements("both", string_search, path, dir_sep, ignore_case, ignore_accents, ignore_folders,
                                 ext_exclude=ext_exclude, ext_include=ext_include, output=output)
    return results


def __get_results(results: list, output_type: str, delimiter: str, search_args: dict,
                  time_start: datetime, time_end: datetime, total_time: float,
                  output_path: str, output_filename):
    """Turns the results list into the selected choice

    Args:
        results (list): List containing all the coincidences found
        output_type (str): Defines how to get the final output {console, txt, csv, json, df, obj_list, list}
            (df = pandas.DataFrame)
        delimiter (str): Delimiter used for {txt, csv, json} output
        search_args (dict): Arguments used for searching. Will be stored in file if output_type = 'json'
        time_start (datetime): Time when search began
        time_end (datetime): Time when search ended
        total_time (float): Difference between time_end and time_start
        output_path (str): Path in which the results file will be located. Default current directory.
        output_filename (str): Name of file in which results will be stored. Default search_results_yyyymmdd_hh_mm_ss 

    Returns:
        object: Returns a boolean if output_type in {'console', 'txt', 'csv', 'json'} 
            and prints the results to the actual choice
            Returns the selected object if output_type in {'obj_list', 'list', 'df'}
            (df = pandas.DataFrame)

    """
    """ if len(results) == 0:
        return 0 """

    output_type = output_type.lower()

    if output_type == "console":
        results.insert(0, HEADER_RESULT)
        max_l = [max(map(custom_utils.find_len, [results[outer_ind][ind] for outer_ind in range(len(results))])) + 1
                 for ind in range(len(results[0]))]

        for result in results:
            print(custom_utils.parse_line(result, max_l))
        return (len(results) - 1)

    elif output_type.lower() in ["df"]:
        return pd.DataFrame(results)
    elif output_type in ["obj_list", "list"]:
        return results
    else:
        output_fullpath = custom_utils.forcedir_sep(os.path.join(
            output_path, f"{output_filename}.{output_type}"), "/")

        if output_type in ("txt", "csv"):
            with open(f"{output_fullpath}", "w", newline="\n", encoding="utf-8") as new_file:
                csv_writer = csv.writer(new_file, delimiter=delimiter)
                headers = list(results[0].keys())
                csv_writer.writerow(headers)

                for result in results:
                    line = list(result.values())
                    csv_writer.writerow(line)

        if output_type in ("json"):
            export_dict = {
                "search_args": search_args,
                "start": time_start,
                "end": time_end,
                "total_time_seconds": total_time,
                "results": results
            }
            with open(f"{output_fullpath}", "w", encoding="utf-8") as new_file:
                json.dump(export_dict, new_file, indent=2, ensure_ascii=False)
        main_logger.info(f"Exported {output_fullpath}")

        return (len(results) - 1)


def go_search(args_dict):
    """Searches for files / folders in the specified 'path'.

    Args:
        args_dict (dict): Dictionary containing all the specification to perform the search.                
            path (str): Root path to look for, default = current directory
            stringsearch (str): The string to look for
            ignorecase (bool): Indicates whether or not searching is case-sensitive
            ignoreaccents (bool): Ignore accents
            regexp (bool): Searches by regular expresion
            ext (list[str]): List of extensions to limit the search
            all (bool): Searches on all folders, ignores 'ignore_folders' switch on 'config.yaml'
            onlyfiles (bool): Searches only for files (not directories)
            onlydirs (bool): Searches only for directories (not files)
            directoryseparator (str): Just for results purposes: which separator use for paths
            output (str): {console,txt,csv,json,df,obj_list,list}
                Indicates if results appears on console or are written to a file
            delimiter (str): Delimiter used for exporting, when output is different than 'console'
            outputpath (str): Path in which the results file will be located. Default current directory.
            outputfilename (str): Name of file in which results will be stored. Default search_results_yyyymmdd_hh_mm_ss 

    Raises:
        NotValidArgumentError: 'onlydirs' and 'onlyfolders' can't be True simultaneously
        NotValidArgumentError: If 'stringsearch' was not specified, 'ext' or 'onlydirs' or 'onlyfiles' must be passed
        NotADirectoryError: Raised if path is not a directory
        NotValidDirectoryError: Raised if path to directory doesn't really exist
    """

    # Compute start time
    time_start = datetime.datetime.now()
    start_search = time.perf_counter()
    main_logger.info(f"Searching for: {args_dict} , please wait...")

    # Checking arguments
    # When called from console, argparse makes validation, when called from other module, it is necessary to check
    ks = args_dict.keys()

    defaults = {"path": os.getcwd(), "stringsearch": "", "ignorecase": False,
                "ignoreaccents": False, "regexp": False, "ext": None, "all": False,
                "onlyfiles": False, "onlydirs": False, "directoryseparator": "/",
                "output": "console", "outputpath": ".", "outputfilename": date_now,
                "delimiter": "\t"}

    for key, value in defaults.items():
        if key not in ks:
            args_dict[key] = value
        elif args_dict[key] is None:
            args_dict[key] = value

    if args_dict["outputpath"]:
        if not os.path.isdir(args_dict["outputpath"]):
            raise NotADirectoryError(
                f"outputpath: {args_dict['outputpath']} is not a valid directory")

    if args_dict["outputfilename"] == "":
        raise MissingFilename("Filename can't be empty if passed as argument")
    else:
        args_dict["outputfilename"] = custom_utils.force_filename(
            args_dict["outputfilename"])

    if (args_dict["onlyfiles"] & args_dict["onlydirs"]):
        raise NotValidArgumentError(
            message="Not valid arguments, forbidden combination: onlyfiles=True and onlydirs=True")

    if args_dict["stringsearch"] == "":
        if ((not args_dict["ext"]) & (not args_dict["onlydirs"]) & (not args_dict["onlyfiles"])):
            raise NotValidArgumentError(message="Not a valid combination to search for:"
                                        "\nUsing 'stringsearch'='' requires specifying 'onlydirs' or ('onlyfiles' and/or 'ext')")

    # The actual searching
    search_results = __search(
        path=args_dict["path"], string_search=args_dict["stringsearch"], ignore_case=args_dict["ignorecase"],
        ignore_accents=args_dict["ignoreaccents"], reg_exp=args_dict["regexp"], ext=args_dict["ext"],
        all=args_dict["all"], only_files=args_dict["onlyfiles"], only_dirs=args_dict["onlydirs"], dir_sep=args_dict["directoryseparator"],
        output=args_dict["output"]
    )

    # Compute time end
    time_end = datetime.datetime.now()
    end_search = time.perf_counter()

    # Compute total time
    time_start_str = time_start.strftime("%Y/%m/%d, %H:%M:%S.%f")
    time_end_str = time_end.strftime("%Y/%m/%d, %H:%M:%S.%f")
    total_time = end_search - start_search

    main_logger.info(
        f"{len(search_results)} coincidences found in {round(total_time, 2)} second(s)")

    # Export / show results
    return __get_results(search_results, args_dict["output"], args_dict["delimiter"], args_dict,
                         time_start_str, time_end_str, total_time, args_dict["outputpath"], args_dict["outputfilename"])


def __config():
    """Reads 'config.yaml' 

    Returns:
        dict: Dictionary with configuration located in 'config.yaml' (ignore_folders + ignore_extensions)

    """
    with open(CONFIG_PATH, "r") as f:
        _config = yaml.safe_load(f)

    return _config


def set_ignore_folders(ignore_list=None):
    """Sets config.yaml 'ignore_folders' list

    Args:
        ignore_list (list[str], optional): List of folders to be ignored in future searches. 
            None of its subfolders will be considered. Defaults to None.
    """
    _config = __config()
    if ignore_list:
        ignore_list = [str(element) for element in ignore_list]
        _config["ignore_folders"] = ignore_list
    else:
        _config["ignore_folders"] = custom_utils.default_ignore_folders()

    with open(CONFIG_PATH, "w+") as yf:
        yaml.dump(_config, yf, allow_unicode=True, default_flow_style=False)


def set_ignore_extensions(ignore_list=None):
    """Sets config.yaml 'ingnore_extension' list

    Args:
        ignore_list (list[str], optional): List of extensions to be ignored in future searches. Defaults to None.
    """
    _config = __config()
    if ignore_list:
        ignore_list = [str(element) for element in ignore_list]
        _config["ignore_extensions"] = ignore_list
    else:
        _config["ignore_extensions"] = []

    with open(CONFIG_PATH, "w+") as yf:
        yaml.dump(_config, yf, allow_unicode=True, default_flow_style=False)



import os
import shutil
import __main__

TEMP_FOLDER_NAME = "__search_tests_dummy_folder__"
ROOT_PATH = os.path.join(os.path.dirname(__main__.__file__), TEMP_FOLDER_NAME)


def drop_test_folder():
    try:
        if os.path.exists(ROOT_PATH):
            shutil.rmtree(ROOT_PATH)
    except Exception as e:
        print(e)


def tear_down_tests():
    drop_test_folder()



import os
import shutil
import __main__

TEMP_FOLDER_NAME = "__search_tests_dummy_folder__"
ROOT_PATH = os.path.join(os.path.dirname(__main__.__file__), TEMP_FOLDER_NAME)


PATH_TREE = {
    "1": "1.mp3",
    "2": "2.mp3",
    "2": "3.mp3",
    "4": "4.mp3",
    "5": "oñe",
    "6": "óne",
    "node_modules": {
        "node_inside1": {
        },
        "node_inside2": {
        },
        "node_inside3": {
        },
        "node_modules": {
            1: "dummy.txt",
            "folder": {
            }
        }
    },
    "one": {
    },
    "oné": {
    },
    "test": {
        "dir1": {
            "1": "one.txt",
            "2": "two.txt",
            "3": "three.txt"
        },
        "dir2": {
            "1": "one.md",
            "2": "two.pdf",
            "3": "three.doc"
        },
        "dir3": {
        }
    },
    "test1": {
        "1": "one.csv",
        "2": "onè.json",
        "3": "oné.csv",
        "4": "twò.txt",
        "5": "twó"
    },
    "test2": {
        "site-packages": {
            "site_inside1": {
                "1": "sp_one",
                "2": "sp_one.json",
                "3": "sp_two.csv",
                "4": "sp_three.txt",
                "5": "sp_four.pdf"
            },
            "site_inside2": {
            },
            "site_inside3": {
            },
            "1": "one",
            "2": "one.json",
            "3": "three.txt",
            "4": "two.csv",
        },
        "test01": {
            "1": "file1.ext",
            "2": "file2.mp3",
            "3": "file3.mp4",
            "4": "one.md",
        },
        "test02": {
            "1": "1.mpg",
            "2": "2.avi",
            "3": "3.mkv",
        },
        "test03": {
        }
    },
    "test3": {
    },
    "trés": {
    },
    "two": {
    }
}


def create_base_folder():
    try:
        if os.path.exists(ROOT_PATH):
            shutil.rmtree(ROOT_PATH)
        os.mkdir(ROOT_PATH)
    except Exception as e:
        print(e)


def create_element(path, tree: dict):
    for key, value in tree.items():
        if isinstance(value, dict):
            os.makedirs(os.path.join(path, key))
            create_element(os.path.join(path, key), value)
        else:
            with open(os.path.join(path, value), "w") as f:
                pass


def create_tree(tree_object):
    create_element(ROOT_PATH, tree_object)


def setup_tests():
    create_base_folder()
    create_tree(PATH_TREE)



import argparse
from findhelp.finder import go_search

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # Arguments for searching
    parser.add_argument("-p", "--path", type=str,
                        help="Root path to look for, default = current directory", default=".")
    parser.add_argument("-s", "--stringsearch", type=str,
                        help="The string to look for", default="")
    parser.add_argument("-i", "--ignorecase", action="store_true",
                        help="Indicates whether or not searching is case-sensitive")
    parser.add_argument("-c", "--ignoreaccents",
                        action="store_true", help="Ignore accents")
    parser.add_argument("-r", "--regexp", action="store_true",
                        help="Searches by regular expresion")
    parser.add_argument("-e", "--ext", nargs="+",
                        help="List of extensions to limit the search")
    parser.add_argument("-a", "--all", action="store_true",
                        help="Searchs on all folders, ignores 'ignore_folders' switch on config.yaml")
    parser.add_argument("-f", "--onlyfiles", action="store_true",
                        help="Searches only for files (not directories)")
    parser.add_argument("-d", "--onlydirs", action="store_true",
                        help="Searches only for directories (not files)")
    parser.add_argument("-ds", "--directoryseparator", help="Just for results purposes: which separator use for paths",
                        type=str, choices=("/", "\\\\", "\\"), default="/")

    # Arguments for outuput results
    parser.add_argument("-o", "--output", choices=("console",
                        "txt", "csv", "json", "df", "obj_list", "list"), default="console", help="Indicates if results appears on console or are written to a file")
    parser.add_argument("-dl", "--delimiter", type=str, default="\t",
                        help="Delimiter used for exporting, when output is different than 'console'")
    parser.add_argument("-m", "--outputpath", type=str,
                        help="Output (results) path", default=".")
    parser.add_argument("-n", "--outputfilename", type=str,
                        help="Output (results) filename (with no extension)")

    # Parse args
    full_args = vars(parser.parse_args())
    print(full_args)

    go_search(full_args)



import unittest
import sys
from findhelp.finder import go_search
from setup_files import setup_tests
from tear_down import tear_down_tests
from findhelp.custom_exceptions import NotValidArgumentError


def main(out=sys.stderr, verbosity=2):
    loader = unittest.TestLoader()

    suite = loader.loadTestsFromModule(sys.modules[__name__])
    unittest.TextTestRunner(out, verbosity=verbosity).run(suite)


class BasicTests(unittest.TestCase):
    # sys.stdout = sys.__stdout__
    # print("Captured", capturedOutput.getvalue())
    @classmethod
    def setUpClass(cls):
        setup_tests()

    @classmethod
    def tearDownClass(cls):
        tear_down_tests()

    def test_search_nothing(self):

        base_dict = {
            "stringsearch": "nothing_to_match",
        }
        result = go_search(base_dict)
        self.assertEqual(result, 0)

    def test_search_one(self):
        base_dict = {
            "path": "./__search_tests_dummy_folder__",
            "stringsearch": "one",
            "output": "list"
        }
        result = go_search(base_dict)

        base_result = [
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]

        self.assertListEqual(sorted(base_result), sorted(result))

    def test_extension(self):
        base_dict = {
            "path": ".",
            "output": "list",
            "ext": ["md", "json"]
        }
        result = go_search(base_dict)

        base_result = [
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'onè.json', '.json'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]

        self.assertListEqual(sorted(base_result), sorted(result))

    def test_dummy_extension(self):
        base_dict = {
            "path": ".",
            "output": "list",
            "ext": ["dummy"]
        }
        result = go_search(base_dict)
        base_result = []
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_only_dirs(self):
        base_dict = {
            "path": ".",
            "stringsearch": "2",
            "output": "list",
            "onlydirs": True
        }
        result = go_search(base_dict)
        base_result = [
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'test2', None, None],
            ['./__search_tests_dummy_folder__/test', 'folder',
                './__search_tests_dummy_folder__', 'dir2', None, None],
            ['./__search_tests_dummy_folder__/test2', 'folder',
                './__search_tests_dummy_folder__', 'test02', None, None]
        ]
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_only_files(self):
        base_dict = {
            "path": ".",
            "stringsearch": "2",
            "output": "list",
            "onlyfiles": True
        }
        result = go_search(base_dict)
        base_result = [
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'file2.mp3', '.mp3'],
            ['./__search_tests_dummy_folder__/test2/test02', 'file',
                './__search_tests_dummy_folder__/test2', 'test02', '2.avi', '.avi']
        ]
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_extension_and_string(self):
        base_dict = {
            "path": ".",
            "stringsearch": "w",
            "output": "list",
            "ext": ["csv", "txt"]
        }
        result = go_search(base_dict)
        base_result = [
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'two.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'twò.txt', '.txt']
        ]
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_no_accents(self):
        base_dict = {
            "path": ".",
            "stringsearch": "one",
            "output": "list",
            "ignoreaccents": True
        }
        result = go_search(base_dict)
        base_result = [
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'oné', None, None],
            ['./__search_tests_dummy_folder__', 'file', '.',
                '__search_tests_dummy_folder__', 'oñe', ''],
            ['./__search_tests_dummy_folder__', 'file', '.',
                '__search_tests_dummy_folder__', 'óne', ''],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'onè.json', '.json'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'oné.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_use_case(self):
        base_dict = {
            "path": ".",
            "stringsearch": "OnE",
            "output": "list"
        }
        result = go_search(base_dict)
        base_result = []
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_ignore_case(self):
        base_dict = {
            "path": ".",
            "stringsearch": "OnE",
            "output": "list",
            "ignorecase": True
        }
        base_result = [
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]
        result = go_search(base_dict)
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_regexp(self):
        base_dict = {
            "path": ".",
            "stringsearch": "o[nñ][eéè].",
            "output": "list",
            "regexp": True
        }
        base_result = [
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'onè.json', '.json'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'oné.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]
        result = go_search(base_dict)
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_include_all(self):
        base_dict = {
            "path": ".",
            "stringsearch": "one",
            "output": "list",
            "all": True
        }
        base_result = [
            ['./__search_tests_dummy_folder__',
                'folder', '.', 'one', None, None],
            ['./__search_tests_dummy_folder__/test/dir1', 'file',
                './__search_tests_dummy_folder__/test', 'dir1', 'one.txt', '.txt'],
            ['./__search_tests_dummy_folder__/test/dir2', 'file',
                './__search_tests_dummy_folder__/test', 'dir2', 'one.md', '.md'],
            ['./__search_tests_dummy_folder__/test1', 'file',
                './__search_tests_dummy_folder__', 'test1', 'one.csv', '.csv'],
            ['./__search_tests_dummy_folder__/test2/site-packages', 'file',
                './__search_tests_dummy_folder__/test2', 'site-packages', 'one', ''],
            ['./__search_tests_dummy_folder__/test2/site-packages', 'file',
                './__search_tests_dummy_folder__/test2', 'site-packages', 'one.json', '.json'],
            ['./__search_tests_dummy_folder__/test2/site-packages/site_inside1', 'file',
                './__search_tests_dummy_folder__/test2/site-packages', 'site_inside1', 'sp_one', ''],
            ['./__search_tests_dummy_folder__/test2/site-packages/site_inside1', 'file',
                './__search_tests_dummy_folder__/test2/site-packages', 'site_inside1', 'sp_one.json', '.json'],
            ['./__search_tests_dummy_folder__/test2/test01', 'file',
                './__search_tests_dummy_folder__/test2', 'test01', 'one.md', '.md']
        ]
        result = go_search(base_dict)
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_include_all2(self):
        base_dict = {
            "path": ".",
            "stringsearch": "sp_two",
            "output": "list",
            "all": True
        }
        base_result = [
            ['./__search_tests_dummy_folder__/test2/site-packages/site_inside1', 'file',
                './__search_tests_dummy_folder__/test2/site-packages', 'site_inside1', 'sp_two.csv', '.csv']
        ]
        result = go_search(base_dict)
        self.assertListEqual(sorted(base_result), sorted(result))

    def test_empty_dict(self):
        with self.assertRaises(NotValidArgumentError):
            base_dict = {}
            go_search(base_dict)

    def test_not_directory(self):
        with self.assertRaises(NotADirectoryError):
            base_dict = {
                "path": "?",
                "stringsearch": "whatever"
            }
            go_search(base_dict)

    def test_non_existent_dir(self):
        with self.assertRaises(NotADirectoryError):
            base_dict = {
                "path": "./dummy_folder/",
                "stringsearch": "whathever"
            }
            go_search(base_dict)


if __name__ == "__main__":
    with open('test_basic_results.txt', 'w') as f:
        main(f)



class NotValidDirectoryError(Exception):
    """Raise exception when directory doesn't exist

    Arguments:
    ----
    Exception (Exception): The base python exception class
    """

    def __init__(self, message):
        """Print out message for this exception.

        Arguments:
        ----
        message (str): Pass in the message returned by the server.
        """
        self.message = message
        super().__init__(self.message)


class NotEqualLengthError(Exception):
    """Raise exception when two lists doesn't have the same length

    Arguments:
    ----
    Exception (Exception): The base python exception class
    """

    def __init__(self, message):
        """Print out message for this exception.

        Arguments:
        ----
        message (str): Pass in the message returned by the server.
        """
        self.message = message
        super().__init__(self.message)


class NotValidArgumentError(Exception):
    """Raise exception when arguments passed are not valid

    Arguments:
    ----
    Exception (Exception): The base python exception class
    """

    def __init__(self, message):
        """Print out message for this exception.

        Arguments:
        ----
        message (str): Pass in the message returned by the server.
        """
        self.message = message
        super().__init__(self.message)


class MissingFilename(Exception):
    """Raise exception when filename is passed as empty string

    Arguments:
    ----
    Exception (Exception): The base python exception class
    """

    def __init__(self, message):
        """Print out message for this exception.

        Arguments:
        ----
        message (str): Pass in the message returned by the server.
        """
        self.message = message
        super().__init__(self.message)



from setuptools import setup


# Load the README file.
with open(file="README.md", mode="r") as readme_handle:
    long_description = readme_handle.read()

setup(
    name='findhelp',
    py_modules=["findhelp"],
    package_dir={"findhelp": "src/findhelp"},
    packages=["findhelp"],
    #package_dir={"": "src"},
    package_data={"src": ["config.yaml"]},
    include_package_data=True,
    author='Miguel Ángel Rendón García',
    author_email='miguerendongarcia@gmail.com',
    #   - MAJOR VERSION 0
    #   - MINOR VERSION 1
    #   - MAINTENANCE VERSION 0
    version='0.1.1',
    description='Helps finding files / directories using custom parameters',

    long_description=long_description,
    long_description_content_type="text/markdown",

    url='https://github.com/rendongarcia/findhelp',

    install_requires=[
        'PyYaml',
        'pandas>=1.0.0',
        'Unidecode>=1.0.22',
    ],

    # Here are the keywords of my library.
    keywords='find, search, file, folder',

    python_requires='>=3.7',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Topic :: Education',
    ]
)



import subprocess
import shutil


subprocess.run(["python", "test_basic.py"])
subprocess.run(["python", "test_config.py"])
subprocess.run(["python", "test_console.py"])
subprocess.run(["python", "test_outputs.py"])

shutil.move("./test_basic_results.txt",
            "../test_results/test_basic_results.txt")
shutil.move("./test_config_results.txt",
            "../test_results/test_config_results.txt")
shutil.move("./test_console_results.txt",
            "../test_results/test_console_results.txt")
shutil.move("./test_output_results.txt",
            "../test_results/test_output_results.txt")
